/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.sinfo.collision;

import java.io.OutputStream;

import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.AbstractCommonExporter;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.JasperReporter;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;

import au.com.bytecode.opencsv.CSVWriter;
import net.sf.jasperreports.engine.JRException;

/**
 * @author Gernot Belger
 */
public final class CollisionExporter extends AbstractCommonExporter<CollisionCalculationResults> {

    @Override
    protected void doWriteCSVData(final CSVWriter writer, final CollisionCalculationResults results) {
        // TODO: Diesen Ablauf in super?

        // TODO: move results into context?
        final ExportContextCSV exportContextCSV = new ExportContextCSV(this.context, writer);

        getLog().info("writeCSVData");

        /* write as csv */
        exportContextCSV.writeCSVGlobalMetadataDefaults(results); // ggf auslagern innerhalb dieser Klasse

        // writer.writeNext(new String[] { "" }); // break line HERE to avoid redundance

        final RiverInfo river = results.getRiver();

        final Class<?> lastResultType = null;

        for (final AbstractCalculationExportableResult<CollisionCalculationResults> result : results.getResults()) {

            final Class<?> resultType = result.getClass();
            if (lastResultType == null || lastResultType != resultType) {
                exportContextCSV.writeBlankLine();
                result.writeCSVHeader(exportContextCSV, results, river);
                exportContextCSV.writeBlankLine();
            } else
                exportContextCSV.writeCSVLine(new String[] { "#" });

            result.writeCsv(exportContextCSV, results);
        }
    }

    @Override
    protected void doWritePdf(final OutputStream out, final CollisionCalculationResults results) {
        // TODO: Move to super
        try {
            final ExportContextCSV exportContextCSV = new ExportContextCSV(this.context, null);

            final JasperReporter reporter = new JasperReporter();

            for (final AbstractCalculationExportableResult<CollisionCalculationResults> result : results.getResults()) {
                final MetaAndTableJRDataSource source = new MetaAndTableJRDataSource();
                getHelper().addJRMetaDataUSINFO(source, results);

                result.addReport(exportContextCSV, results, reporter, source);
            }

            reporter.exportPDF(out);
        }
        catch (final JRException je) {
            getLog().warn("Error generating PDF Report!", je);
        }
    }
}
